@@ -1,7 +1,7 @@
 """Controller module."""
 
-import os.path
-from typing import List, Dict
+import os
+from typing import List, Dict, Any
 from uuid import uuid4
 
 from flowcept.commons.autoflush_buffer import AutoflushBuffer
@@ -10,7 +10,7 @@ from flowcept.commons.flowcept_dataclasses.workflow_object import (
     WorkflowObject,
 )
 from flowcept.commons.flowcept_logger import FlowceptLogger
-from flowcept.commons.utils import ClassProperty
+from flowcept.commons.utils import ClassProperty, buffer_to_disk
 from flowcept.configs import (
     MQ_INSTANCES,
     INSTRUMENTATION_ENABLED,
@@ -45,7 +45,7 @@ class Flowcept(object):
     def __init__(
         self,
         interceptors: List[str] = None,
-        bundle_exec_id=None,
+        bundle_exec_id: str = None,
         campaign_id: str = None,
         workflow_id: str = None,
         workflow_name: str = None,
@@ -53,6 +53,7 @@ class Flowcept(object):
         start_persistence=True,
         check_safe_stops=True,  # TODO add to docstring
         save_workflow=True,
+        delete_buffer_file=True,
         *args,
         **kwargs,
     ):
@@ -69,7 +70,7 @@ class Flowcept(object):
             Examples: "instrumentation", "dask", "mlflow", ...
             The order of interceptors matters — place the outer-most interceptor first,
 
-        bundle_exec_id : Any, optional
+        bundle_exec_id : str, optional
             Identifier for grouping interceptors in a bundle, essential for the correct initialization and stop of
             interceptors. If not provided, a unique ID is assigned.
 
@@ -91,6 +92,9 @@ class Flowcept(object):
         save_workflow : bool, default=True
             If True, a workflow object message is sent.
 
+        delete_buffer_file : bool, default=True
+            if True, deletes an existing existing buffer file or ignores if it doesn't exist.
+
         Additional arguments (`*args`, `**kwargs`) are used for specific adapters.
             For example, when using the Dask interceptor, the `dask_client` argument
             should be provided in `kwargs` to enable saving the Dask workflow, which is recommended.
@@ -102,9 +106,9 @@ class Flowcept(object):
         self.buffer = None
         self._check_safe_stops = check_safe_stops
         if bundle_exec_id is None:
-            self._bundle_exec_id = id(self)
+            self.bundle_exec_id = str(id(self))
         else:
-            self._bundle_exec_id = bundle_exec_id
+            self.bundle_exec_id = str(bundle_exec_id)
 
         self.enabled = True
         self.is_started = False
@@ -130,8 +134,11 @@ class Flowcept(object):
         self.workflow_name = workflow_name
         self.workflow_args = workflow_args
 
+        if delete_buffer_file:
+            Flowcept.delete_buffer_file()
+
     def start(self):
-        """Start it."""
+        """Start Flowcept Controller."""
         if self.is_started or not self.enabled:
             self.logger.warning("DB inserter may be already started or instrumentation is not set")
             return self
@@ -155,7 +162,7 @@ class Flowcept(object):
                 Flowcept.current_workflow_id = self.current_workflow_id
 
                 interceptor_inst = BaseInterceptor.build(interceptor)
-                interceptor_inst.start(bundle_exec_id=self._bundle_exec_id, check_safe_stops=self._check_safe_stops)
+                interceptor_inst.start(bundle_exec_id=self.bundle_exec_id, check_safe_stops=self._check_safe_stops)
                 self._interceptor_instances.append(interceptor_inst)
                 if isinstance(interceptor_inst._mq_dao.buffer, AutoflushBuffer):
                     Flowcept.buffer = self.buffer = interceptor_inst._mq_dao.buffer.current_buffer
@@ -171,29 +178,115 @@ class Flowcept(object):
         self.logger.debug("Flowcept started successfully.")
         return self
 
+    def get_buffer(self, return_df: bool = False):
+        """
+        Retrieve the in-memory message buffer.
+
+        Parameters
+        ----------
+        return_df : bool, optional
+            If False (default), return the raw buffer as a list of dictionaries.
+            If True, normalize the buffer into a pandas DataFrame with dotted
+            notation for nested keys. Requires ``pandas`` to be installed.
+
+        Returns
+        -------
+        list of dict or pandas.DataFrame
+            - If ``return_df=False``: the buffer as a list of dictionaries.
+            - If ``return_df=True``: the buffer as a normalized DataFrame.
+
+        Raises
+        ------
+        ModuleNotFoundError
+            If ``return_df=True`` but ``pandas`` is not installed.
+
+        Examples
+        --------
+        >>> buf = flowcept.get_buffer()
+        >>> isinstance(buf, list)
+        True
+
+        >>> df = flowcept.get_buffer(return_df=True)
+        >>> "generated.attention" in df.columns
+        True
+        """
+        if return_df:
+            try:
+                import pandas as pd
+            except ModuleNotFoundError as e:
+                raise ModuleNotFoundError("pandas is required when return_df=True. Please install pandas.") from e
+            return pd.json_normalize(self.buffer, sep=".")
+        return self.buffer
+
     def _publish_buffer(self):
         self._interceptor_instances[0]._mq_dao.bulk_publish(self.buffer)
 
+    def dump_buffer(self, path: str = None):
+        """
+        Dump the current in-memory buffer to a JSON Lines (JSONL) file.
+
+        Each element of the buffer (a dictionary) is serialized as a single line
+        of JSON. If no path is provided, the default path from the settings file
+        is used.
+
+        Parameters
+        ----------
+        path : str, optional
+            Destination file path for the JSONL output. If not provided,
+            defaults to ``DUMP_BUFFER_PATH`` as configured in the settings.
+
+        Returns
+        -------
+        None
+            The buffer is written to disk, no value is returned.
+
+        Notes
+        -----
+        - The buffer is expected to be a list of dictionaries.
+        - Existing files at the specified path will be overwritten.
+        - Logging is performed through the class logger.
+
+        Examples
+        --------
+        >>> flowcept.dump_buffer("buffer.jsonl")
+        # Writes buffer contents to buffer.jsonl
+
+        >>> flowcept.dump_buffer()
+        # Writes buffer contents to the default path defined in settings
+        """
+        if path is None:
+            path = DUMP_BUFFER_PATH
+        buffer_to_disk(self.buffer, path, self.logger)
+
     @staticmethod
-    def read_messages_file(file_path: str = None) -> List[Dict]:
+    def read_buffer_file(file_path: str | None = None, return_df: bool = False, normalize_df: bool = False):
         """
         Read a JSON Lines (JSONL) file containing captured Flowcept messages.
 
         This function loads a file where each line is a serialized JSON object.
         It joins the lines into a single JSON array and parses them efficiently
-        with ``orjson``.
+        with ``orjson``. If ``return_df`` is True, it returns a pandas DataFrame
+        created via ``pandas.json_normalize(..., sep='.')`` so nested fields become
+        dot-separated columns (for example, ``generated.attention``).
 
         Parameters
         ----------
         file_path : str, optional
-            Path to the messages file. If not provided, defaults to the
-            value of ``DUMP_BUFFER_PATH`` from the configuration.
-            If neither is provided, an assertion error is raised.
+            Path to the buffer file. If not provided, defaults to the value of
+            ``DUMP_BUFFER_PATH`` from the configuration. If neither is provided,
+            an assertion error is raised.
+        return_df : bool, default False
+            If True, return a normalized pandas DataFrame. If False, return the
+            parsed list of dictionaries.
+        normalize_df: bool, default False
+            If True, normalize the inner dicts (e.g., used, generated, custom_metadata) as individual columns in the
+            returned DataFrame.
 
         Returns
         -------
-        List[dict]
-            A list of message objects (dictionaries) parsed from the file.
+        list of dict or pandas.DataFrame
+            A list of message objects when ``return_df`` is False,
+            otherwise a normalized DataFrame with dot-separated columns.
 
         Raises
         ------
@@ -203,35 +296,92 @@ class Flowcept(object):
             If the specified file does not exist.
         orjson.JSONDecodeError
             If the file contents cannot be parsed as valid JSON.
+        ModuleNotFoundError
+            If ``return_df`` is True but pandas is not installed.
 
         Examples
         --------
-        Read messages from a file explicitly:
+        Read messages as a list:
 
-        >>> msgs = read_messages_file("offline_buffer.jsonl")
-        >>> print(len(msgs))
-        128
+        >>> msgs = read_buffer_file("offline_buffer.jsonl")
+        >>> len(msgs) > 0
+        True
 
-        Use the default dump buffer path from config:
+        Read messages as a normalized DataFrame:
 
-        >>> msgs = read_messages_file()
-        >>> for m in msgs[:2]:
-        ...     print(m["type"], m.get("workflow_id"))
-        task_start wf_123
-        task_end wf_123
+        >>> df = read_buffer_file("offline_buffer.jsonl", return_df=True)
+        >>> "generated.attention" in df.columns
+        True
         """
+        import os
         import orjson
 
-        _buffer = []
         if file_path is None:
             file_path = DUMP_BUFFER_PATH
         assert file_path is not None, "Please indicate file_path either in the argument or in the config file."
         if not os.path.exists(file_path):
-            raise f"File {file_path} has not been created. It will only be created if you run in fully offline mode."
+            raise FileNotFoundError(f"Flowcept buffer file '{file_path}' was not found.")
+
         with open(file_path, "rb") as f:
             lines = [ln for ln in f.read().splitlines() if ln]
-        _buffer = orjson.loads(b"[" + b",".join(lines) + b"]")
-        return _buffer
+
+        buffer: List[Dict[str, Any]] = orjson.loads(b"[" + b",".join(lines) + b"]")
+
+        if return_df:
+            try:
+                import pandas as pd
+            except ModuleNotFoundError as e:
+                raise ModuleNotFoundError("pandas is required when return_df=True. Please install pandas.") from e
+            if normalize_df:
+                return pd.json_normalize(buffer, sep=".")
+            else:
+                return pd.read_json(file_path, lines=True)
+
+        return buffer
+
+    @staticmethod
+    def delete_buffer_file(path: str = None):
+        """
+        Delete the buffer file from disk if it exists.
+
+        If no path is provided, the default path from the settings file
+        is used. Logs whether the file was successfully removed or not found.
+
+        Parameters
+        ----------
+        path : str, optional
+            Path to the buffer JSONL file. If not provided,
+            defaults to ``DUMP_BUFFER_PATH`` as configured in the settings.
+
+        Returns
+        -------
+        None
+            The file is deleted from disk if it exists, no value is returned.
+
+        Notes
+        -----
+        - This operation only affects the file on disk. It does not clear
+          the in-memory buffer.
+        - Logging is performed through the class logger.
+
+        Examples
+        --------
+        >>> flowcept.delete_buffer_file("buffer.jsonl")
+        # Deletes buffer.jsonl if it exists
+
+        >>> flowcept.delete_buffer_file()
+        # Deletes the default buffer file defined in settings
+        """
+        if path is None:
+            path = DUMP_BUFFER_PATH
+
+        try:
+            if os.path.exists(path):
+                os.remove(path)
+                FlowceptLogger().info(f"Buffer file deleted: {path}")
+        except Exception as e:
+            FlowceptLogger().error(f"Failed to delete buffer file: {path}")
+            FlowceptLogger().exception(e)
 
     def save_workflow(self, interceptor: str, interceptor_instance: BaseInterceptor):
         """
@@ -282,12 +432,12 @@ class Flowcept(object):
 
         from flowcept.flowceptor.consumers.document_inserter import DocumentInserter
 
-        doc_inserter = DocumentInserter(check_safe_stops=self._check_safe_stops, bundle_exec_id=self._bundle_exec_id)
+        doc_inserter = DocumentInserter(check_safe_stops=self._check_safe_stops, bundle_exec_id=self.bundle_exec_id)
         doc_inserter.start()
         self._db_inserters.append(doc_inserter)
 
     def stop(self):
-        """Stop it."""
+        """Stop Flowcept controller."""
         if not self.is_started or not self.enabled:
             self.logger.warning("Flowcept is already stopped or may never have been started!")
             return
@@ -301,7 +451,7 @@ class Flowcept(object):
         if len(self._db_inserters):
             self.logger.info("Stopping DB Inserters...")
             for db_inserter in self._db_inserters:
-                db_inserter.stop(bundle_exec_id=self._bundle_exec_id)
+                db_inserter.stop(bundle_exec_id=self.bundle_exec_id)
 
         Flowcept.buffer = self.buffer = None
         self.is_started = False
@@ -331,8 +481,7 @@ class Flowcept(object):
         - The method tests the liveness of the MQ service using `MQDao`.
         - If `MONGO_ENABLED` is True, it also checks the liveness of the MongoDB service
           using `MongoDBDAO`.
-        - Logs errors if any service is not ready, and logs success when both services are
-        operational.
+        - Logs errors if any service is not ready, and logs success when both services are operational.
 
         Examples
         --------
